#initialized package and sets for parallelization
#install.packages("doParallel")
#library(doParallel)
#registerDoParallel(makeCluster(detectCores()-1))

#increase memory limits
memory.limit(17592186044415)
memory.size(max=T)

#set working directory
setwd("D:/Daten/Agent-based modelling/final files")

library(rlist)
migration <- list.load("migration.rdata")
admixture <- read.csv("admixture estimates.csv", header=T)
history   <- read.csv("history.csv", header=T)

#death rates
#(from Pietrusewsky, M, Douglas, MT, & Ikehara-Quebral, RM (1997) An Assessment of Health and Disease in the Prehistoric Inhabitants of the Mariana Islands. Am. J. Phys. Anthropol. 104: 315 - 342.)
death.rate <- matrix(ncol=2, nrow=56)
colnames(death.rate) <- c("age", "death rate")
death.rate[,1] <- 0:55
death.rate[,2] <- c(0.02, 0.035, 0.0476, 0.07, 0.09, 0.1, 0.1032, 0.1032, 0.093, 0.072, 0.05, 0.047, 0.046, 0.0443, 0.048, 0.055, 0.062, 0.064, 0.0648, 0.073, 0.08, 0.09, 0.093, 0.098, 0.123, 0.145, 0.167, 0.18, 0.1965, 0.198, 0.1995, 0.201, 0.202, 0.2036, 0.23, 0.25, 0.268, 0.27, 0.2757, 0.278, 0.28, 0.283, 0.285, 0.286, 0.29, 0.31, 0.37, 0.42, 0.5, 0.62, 0.73, 0.84, 0.89, 0.94, 0.97, 1)

survival <- cumsum(death.rate[,2])
survival <- max(survival)-survival
survival <- survival/sum(survival)
survival <- survival[1:55]

death.rate <- c(0, (-1)*diff(survival/survival[1]), 1)

#birth rate
#birth probability by age
birth.rate <- matrix(ncol=2, nrow=56)
colnames(birth.rate) <- c("age", "birth rate")
birth.rate[,1] <- 0:55
birth.rate[,2] <- c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.001, 0.003, 0.01, 0.04, 0.09, 0.13, 0.17, 0.24, 0.27, 0.29, 0.31, 0.31, 0.295, 0.28, 0.265, 0.245, 0.225, 0.205, 0.19, 0.17, 0.15, 0.13, 0.11, 0.09, 0.075, 0.06, 0.045, 0.03, 0.01, 0.007, 0.0055, 0.0035, 0.0025, 0.0024, 0.0022, 0.0015, 0.001, 0.0005, 0.0001, 0, 0, 0, 0, 0)

#create matrix runsummary
asian.fecundities   <- c(3, 4, 5, 6, 7)
papuan.fecundities  <- c(3, 4, 5, 6, 7)
asian.migrations    <- c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8)
papuan.migrations   <- c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8)
marriage.weights    <- c(0.00, 0.05, 0.10, 0.15, 0.20, 0.25)
migration.distances <- c(0.2, 0.4, 0.6, 0.8, 1.0)

runsummary <- expand.grid(asian.fecundities, papuan.fecundities, asian.migrations, papuan.migrations, marriage.weights, migration.distances)
runsummary <- do.call(rbind, replicate(100, runsummary, simplify=FALSE))
runsummary <- cbind(1:nrow(runsummary), runsummary, matrix(NA, nrow=nrow(runsummary), ncol=3))
colnames(runsummary) <- c("runID", "asian fecundity", "papuan fecundity", "asian migration", "papuan migration", "marriage weight", "migration distances", "Asian autosome ancestry", "Asian mt ancestry", "Asian Y ancestry")

for (i in 1:nrow(runsummary)) {
  
  #initialization
  runlist <- as.data.frame(matrix(nrow=26000000, ncol=17))
  colnames(runlist) <- c("ID", "sex", "birth year", "birth place", "death year", "death place", "residence", "year of migration", "partner", "parent1", "parent2", "autosomal Asian ancestry", "mitochondrial Asian ancestry", "Y Asian ancestry", "overall ancestry", "max. offspring", "time of migration")
  runlist[1:21000,1] <- 1:21000
  runlist[1:21000,2] <- sample(c("m", "f"), 21000, replace=T)
  runlist[1:21000,3] <- (-1) * sample(0:54, 21000, prob=survival, replace=T)
  runlist[1:21000,4] <- rep(which(history$initial.settlement==0),each=120)
  runlist[,7] <- runlist[,4]
    group <- history$initial.group[runlist[,4]]
  runlist[,12] <- ifelse(group=="Asian", 1, 0)
  runlist[,13] <- ifelse(group=="Asian", 1, 0)
  runlist[,14] <- ifelse(group=="Asian", 1, 0)
  runlist[,15] <- ifelse(group=="Asian", 1, 0)
    females <- which(runlist[,2]=="f")
    asian   <- which(runlist[,15] > 0.5)
    papuan  <- which(runlist[,15] < 0.5)
  runlist[females[females %in% asian],16] <- rpois(sum(females %in% asian), runsummary[i,2])
  runlist[females[females %in% papuan],16] <- rpois(sum(females %in% papuan), runsummary[i,3])
  
  autosomes <- matrix(nrow=21000, ncol=26)
  colnames(autosomes) <- c("ID", "auto1", "auto2", "auto3", "auto4", "auto5", "auto6", "auto7", "auto8", "auto9", "auto10", "auto11", "auto12", "auto13", "auto14", "auto15", "auto16", "auto17", "auto18", "auto19", "auto20", "auto21", "auto22", "auto23", "auto24", "auto25")
  autosomes[,1]    <- 1:21000
    group <- ifelse(group[1:21000]=="Asian", 1, 0)
  autosomes[,2:26] <- rep(group, 25)
  
  #run simulations (actions for one year)
  for (t in 0:4500) {
    
    carrying.capacity <- 120*1.0003^t
    
    #death
    for (j in 2:56) {
      alive <- which(is.na(runlist[,5]))
      candidates <- which(t-runlist[,3]==j-1)
      candidates <- candidates[candidates %in% alive]
      deaths <- candidates[which(sample(c(0,1), length(candidates), prob=c(1-death.rate[j], death.rate[j]), replace=T)==1)]
      if (length(deaths)>0) {
        runlist[deaths,5] <- t
        runlist[deaths,6] <- runlist[deaths,7]
        partner <- which(!is.na(runlist[deaths,9]))
        if (length(partner)>0) {
          mates <- runlist[deaths,9]
          runlist[deaths,9] <- NA
          runlist[mates,9] <- NA
        }
        autosomes <- autosomes[-deaths,]
      }
    }
    
    #migration
    subset <- runlist[which(t - runlist[,3] > 11),]  #subsetting age > 11
    subset <- subset[which(is.na(subset[,5])),]      #subsetting alive
    subset <- subset[which(is.na(subset[,9])),]      #subsetting uncoupled
    subset <- subset[which(subset[,4]==subset[,7]),] #subsetting not yet migrated
    subset <- subset[sample(nrow(subset)),]          #randomizing order of individuals
    for (j in 1:nrow(subset)) {
      #iterate through all individuals in random order
      if (runif(1) < ifelse(subset[j,15]>0.5, runsummary[i,4], runsummary[i,5])) {
        #conditional test for migration (based on Asian/Papuan migration rates)
        deme.candidates <- migration[[subset[j,7]]][,1]
        #test for historical reachability
        possible <- NULL
        for (k in 1:length(deme.candidates)) {
          #selection of reachable demes
          if ((!is.na(history[deme.candidates[k],1]) || t >= history[deme.candidates[k],2])) {
            #is deme historically reachable?
            if (sum(is.na(runlist[which(runlist[,7]==subset[j,7]),5])) <= round(carrying.capacity) - 1) {
              #is deme below its carrying capacity?
              if (subset[j,15] < 0.5) {
                #for Papuan ancestries: is deme within reachable distance?
                if (migration[[subset[j,7]]][k,4] < runsummary[i,5]) {
                  possible <- c(possible, deme.candidates[k])
                }
              }
              if (subset[j,15] > 0.5) {
                possible <- c(possible, deme.candidates[k])
              }
            }
          }
        }
        dists <- migration[[subset[j,7]]][which(migration[[subset[j,7]]][,1] %in% possible),4]
        prob <- dists/sum(dists)
        if (length(possible) > 0) {
          new.deme <- sample(possible, 1, prob=prob)
          runlist[subset[j,1],6] <- new.deme
          runlist[subset[j,1],7] <- new.deme
        }
      }
      print(c(t,j))
    }
    
    #mating
    #fr alle frauen > 11 a
    #nicht verheiratet
    #suche partner in dem mit papuan/asian prferenz
    #heirate
    
    #offspring
    #wenn bereits verheiratet
    #und zahl der gezeugten Kinder < zahl mglicher kinder
    #generiere kind
    #zufllige whl der autosomen von mutter & vater
    
  }
  
  #hier mittlere Distanz zwischen Simulation und Realitt berechnen und in runsummary einfgen
  
  write.csv(runlist, paste("runlist", i, ".csv", sep=""), row.names=F)
  write.csv(runsummary, "runsummary.csv", row.names=F)
  
  print(t)
  
}

